import 'package:flutter/material.dart';
import 'package:awesome_notifications/awesome_notifications.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:http/http.dart' as http;
import 'package:firebase_core/firebase_core.dart';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'dart:convert';
import 'dart:io';

String host = 'https://app.nativenotify.com';
var pushDataObject;

class NativeNotify {
  static void initialize(
      flutterAppId, 
      flutterAppToken, 
      firebaseServerKey,
      notificationIcon) async {
    final prefs = await SharedPreferences.getInstance();

    await prefs.setInt('flutterAppId', flutterAppId);
    await prefs.setString('flutterAppToken', flutterAppToken);

    final flutterDeviceId = prefs.getInt('flutterDeviceId');
    final firebaseServerKeyShared = prefs.getString('firebaseServerKey');

    if (firebaseServerKey == null) {
      if (flutterDeviceId == null) {
        try {
          var response = await http.post(Uri.parse('$host/api/flutter/device'),
              headers: {
                HttpHeaders.contentTypeHeader: 'application/json',
              },
              body: jsonEncode({
                'flutterAppId': '$flutterAppId',
                'flutterAppToken': flutterAppToken
              }));

          var jsonData = jsonDecode(response.body);

          await jsonData.forEach((key, value) async => {
                if (key == 'flutter_device_id')
                  {await prefs.setInt('flutterDeviceId', value)}
              });

          print(
              "You can send Native Notify push notifications now. You're registered!");

          getPushNotification();
        } catch (e) {
          // print(e);
        }
      } else {
        getPushNotification();
      }
    } else {
      String? firebaseToken;

      try {
        await Firebase.initializeApp();
        firebaseToken = await FirebaseMessaging.instance.getToken();
      } catch (e) {
        firebaseToken = null;
      }

      if (flutterDeviceId == null ||
          firebaseServerKeyShared == 'your-firebase-server-key' ||
          firebaseServerKeyShared == null ||
          firebaseServerKeyShared == '') {
        if (firebaseServerKey != null) {
          await prefs.setString('firebaseServerKey', firebaseServerKey);
        }

        try {
          var response = await http.post(Uri.parse('$host/api/flutter/device'),
              headers: {
                HttpHeaders.contentTypeHeader: 'application/json',
              },
              body: jsonEncode({
                'flutterAppId': '$flutterAppId',
                'flutterAppToken': flutterAppToken,
                'firebaseToken': firebaseToken,
                'firebaseServerKey': firebaseServerKey
              }));

          var jsonData = jsonDecode(response.body);

          await jsonData.forEach((key, value) async => {
                if (key == 'flutter_device_id')
                  {await prefs.setInt('flutterDeviceId', value)}
              });

          print(
              "You can send Native Notify push notifications now. You're registered!");

          if (firebaseToken == null) {
            getPushNotification();
          }
        } catch (e) {
          // print(e);
        }
      }

      if (firebaseToken == null) {
        getPushNotification();
      }
    }

    if (notificationIcon == null) {
      AwesomeNotifications().initialize(
        null, 
        [
          NotificationChannel(
              channelKey: 'mass',
              channelName: 'Native Notify Mass Notifications',
              channelDescription: "Notifications for all users",
              defaultColor: Color.fromARGB(255, 14, 14, 14),
              ledColor: Colors.white,
              playSound: true,
              enableLights: true,
              enableVibration: true,
              importance: NotificationImportance.High)
        ]
      );
    } else {
      AwesomeNotifications().initialize(
        notificationIcon, 
        [
          NotificationChannel(
              channelKey: 'mass',
              channelName: 'Native Notify Mass Notifications',
              channelDescription: "Notifications for all users",
              defaultColor: Color(0xFF171718),
              ledColor: Colors.white,
              playSound: true,
              enableLights: true,
              enableVibration: true,
              importance: NotificationImportance.High)
        ]
      );
    }

    AwesomeNotifications().isNotificationAllowed().then((isAllowed) {
      if (!isAllowed) {
        AwesomeNotifications().requestPermissionToSendNotifications();
      }
    });

    FirebaseMessaging.onMessage.listen(_firebaseMessagingHandler);
    FirebaseMessaging.onBackgroundMessage(_firebaseMessagingHandler);
  }

  static void registerIndieID(indieSubID) async {
    final prefs = await SharedPreferences.getInstance();

    final flutterAppId = prefs.getInt('flutterAppId');
    final flutterDeviceId = prefs.getInt('flutterDeviceId');

    try {
     await http.put(Uri.parse('$host/api/flutter/device/indie/id'),
              headers: {
                HttpHeaders.contentTypeHeader: 'application/json',
              },
              body: jsonEncode({
                'flutter_app_id': '$flutterAppId',
                'flutter_device_id': '$flutterDeviceId',
                'indie_push_sub_id': indieSubID
              }));
    } catch (e) {
      // print(e);
    }
  }

  static void sendIndieNotification(flutterAppId, flutterAppToken, indieSubID,
      title, body, bigPictureURL, dataObject) async {
    try {
      await http.post(Uri.parse('$host/api/flutter/notification'),
          headers: {
            HttpHeaders.contentTypeHeader: 'application/json',
          },
          body: jsonEncode({
            'flutterAppId': '$flutterAppId',
            'flutterAppToken': flutterAppToken,
            'indieSubID': indieSubID,
            'title': title,
            'body': body,
            'bigPictureURL': bigPictureURL,
            'data': dataObject,
          }));
    } catch (e) {
      // print(e);
    }
  }
}

Future<void> _firebaseMessagingHandler(RemoteMessage message) async {
  await Firebase.initializeApp();

  var deviceInfo = await jsonDecode(message.data['deviceInfo']);

  var dataObject = await jsonDecode(message.data['data']);
  pushDataObject = dataObject;

  final flutterDeviceId = deviceInfo['flutterDeviceId'];
  final flutterAppId = deviceInfo['flutterAppId'];
  final flutterAppToken = deviceInfo['flutterAppToken'];

  try {
    await http.read(Uri.parse(
        '$host/api/flutter/notification/$flutterDeviceId/$flutterAppId/$flutterAppToken'));
  } catch (e) {
    // print(e);
  }

  AwesomeNotifications().createNotificationFromJsonData(message.data);
}

// registerIndieSubID
// sendIndiePushNotification

void checkForNotifications() async {
  final prefs = await SharedPreferences.getInstance();

  final flutterDeviceId = prefs.getInt('flutterDeviceId');
  final flutterAppId = prefs.getInt('flutterAppId');
  final flutterAppToken = prefs.getString('flutterAppToken');

  try {
    var response = await http.read(Uri.parse(
        '$host/api/flutter/notification/$flutterDeviceId/$flutterAppId/$flutterAppToken'));

    List<dynamic> notifications = jsonDecode(response);

    for (var i = 0; i < notifications.length; i++) {
      var dataObject =
          await jsonDecode(notifications[i]['notification_data_object']);
      pushDataObject = dataObject;

      if (notifications[i]['notification_big_picture_url'] == null ||
          notifications[i]['notification_big_picture_url'] == '') {
        AwesomeNotifications().createNotification(
          content: NotificationContent(
              id: notifications[i]['flutter_notification_id'],
              channelKey: 'mass',
              title: notifications[i]['notification_title'],
              body: notifications[i]['notification_body']),
        );
      } else {
        AwesomeNotifications().createNotification(
          content: NotificationContent(
              id: notifications[i]['flutter_notification_id'],
              channelKey: 'mass',
              title: notifications[i]['notification_title'],
              body: notifications[i]['notification_body'],
              bigPicture: notifications[i]['notification_big_picture_url'],
              notificationLayout: NotificationLayout.BigPicture),
        );
      }
    }
  } catch (e) {
    // print(e);
  }
}

void getPushNotification() {
  Future.delayed(const Duration(seconds: 30), () {
    return checkForNotifications();
  }).whenComplete(() => getPushNotification());
}
